@@ -4,7 +4,7 @@
 
 ;; Author: Milan Glacier <dev@milanglacier.com>
 ;; Maintainer: Milan Glacier <dev@milanglacier.com>
-;; Version: 0.5.3
+;; Version: 0.5.4
 ;; URL: https://github.com/milanglacier/minuet-ai.el
 ;; Package-Requires: ((emacs "29") (plz "0.9") (dash "2.19.1"))
 
@@ -141,6 +141,14 @@ When non-nil and a completion item has multiple lines, create another
 completion item containing only its first line."
   :type 'boolean)
 
+(defcustom minuet-show-error-message-on-minibuffer nil
+  "Whether to show the error messages in minibuffer.
+When non-nil, if a request fails or times out without generating even
+a single token, the error message will be shown in the minibuffer.
+Note that you can always inspect `minuet-buffer-name' to view the
+complete error log."
+  :type 'boolean)
+
 (defcustom minuet-after-cursor-filter-length 15
   "Length of context after cursor used to filter completion text.
 
@@ -426,25 +434,37 @@ Also cancel any pending requests unless NO-CANCEL is t."
               (index (or index 0))
               (total (length suggestions))
               (suggestion (nth index suggestions))
-              ;; Ensure the overlay appears after the cursor If
-              ;; point is not at end-of-line, offset the overlay
-              ;; position by 1
-              (ov-point (if (eolp) (point) (1+ (point))))
-              (ov (make-overlay ov-point ov-point)))
+              ;; 'Display' is used when not at the end-of-line to
+              ;; ensure proper overlay positioning. Other methods,
+              ;; such as `after-string' or `before-string', fail to
+              ;; correctly position the cursor (which should precede
+              ;; the overlay) and the overlay itself.
+              (ov-method (if (eolp) 'after-string 'display))
+              (ov-start (point))
+              (ov-end (if (eq ov-method 'display) (1+ ov-start) ov-start))
+              ;; When using 'display', we include the character next
+              ;; to the current point into the overlay to ensure its
+              ;; visibility, as the overlay otherwise conceals it.
+              (offset-char (if (eq ov-method 'after-string)
+                               ""
+                             (buffer-substring ov-start ov-end)))
+              (ov (make-overlay ov-start ov-end)))
     (setq minuet--current-suggestions suggestions
           minuet--current-suggestion-index index
-          minuet--last-point (point))
+          minuet--last-point ov-start)
     ;; HACK: Adapted from copilot.el We add a 'cursor text property to the
     ;; first character of the suggestion to simulate the visual effect of
     ;; placing the overlay after the cursor
     (put-text-property 0 1 'cursor t suggestion)
-    (overlay-put ov 'after-string
-                 (propertize
-                  (format "%s%s"
-                          suggestion
-                          (if (= total minuet-n-completions 1) ""
-                            (format " (%d/%d)" (1+ index) total)))
-                  'face 'minuet-suggestion-face))
+    (overlay-put ov ov-method
+                 (concat
+                  (propertize
+                   (format "%s%s"
+                           suggestion
+                           (if (= total minuet-n-completions 1) ""
+                             (format " (%d/%d)" (1+ index) total)))
+                   'face 'minuet-suggestion-face)
+                  offset-char))
     (overlay-put ov 'minuet t)
     (setq minuet--current-overlay ov)
     (minuet-active-mode 1)))
@@ -498,8 +518,9 @@ Also print the MESSAGE when MESSAGE-P is t."
   (with-current-buffer (get-buffer-create minuet-buffer-name)
     (goto-char (point-max))
     (insert (format "%s %s\n" message (format-time-string "%Y-%02m-%02d %02H:%02M:%02S")))
-    (when message-p
-      (message "%s" message))))
+    (when message-p (message "%s" message))
+    ;; make sure this function returns nil
+    nil))
 
 (defun minuet--add-tab-comment ()
   "Add comment string for tab use into the prompt."
@@ -651,19 +672,21 @@ The filter sequence is obtained from CONTEXT."
   (setq response (split-string response "[\r]?\n"))
   (let (result)
     (dolist (line response)
-      (if-let* ((json (ignore-errors
-                        (json-parse-string
-                         (replace-regexp-in-string "^data: " "" line)
-                         :object-type 'plist :array-type 'list)))
-                (text (ignore-errors
-                        (funcall get-text-fn json))))
-          (when (and (stringp text)
-                     (not (equal text "")))
-            (push text result))))
+      (when-let* ((json (ignore-errors
+                          (json-parse-string
+                           (replace-regexp-in-string "^data: " "" line)
+                           :object-type 'plist :array-type 'list)))
+                  (text (ignore-errors
+                          (funcall get-text-fn json))))
+        (when (and (stringp text)
+                   (not (equal text "")))
+          (push text result))))
     (setq result (apply #'concat (nreverse result)))
     (if (equal result "")
-        (progn (minuet--log (format "Minuet returns no text for streaming: %s" response))
-               nil)
+        (progn
+          (minuet--log "Minuet: Stream decoding failed for response:"
+                       minuet-show-error-message-on-minibuffer)
+          (minuet--log response))
       result)))
 
 (defmacro minuet--make-process-stream-filter (response)
@@ -679,9 +702,11 @@ The filter sequence is obtained from CONTEXT."
 
 RESPONSE will be stored in the temp variable create by
 `minuet--make-process-stream-filter' parsed by GET-TEXT-FN."
-  (when-let* ((response (nreverse response))
-              (response (apply #'concat response)))
-    (minuet--stream-decode response get-text-fn)))
+  (if-let* ((response (nreverse response))
+            (response (apply #'concat response)))
+      (minuet--stream-decode response get-text-fn)
+    (minuet--log "Minuet: Empty stream response - no data received"
+                 minuet-show-error-message-on-minibuffer)))
 
 (defun minuet--handle-chat-completion-timeout (context err response get-text-fn name callback)
   "Handle the timeout error for chat completion.
@@ -699,7 +724,8 @@ the errors."
                                        context))
                     (completion-items (minuet--remove-spaces completion-items)))
           (funcall callback completion-items)))
-    (minuet--log (format "An error occured when sending request to %s" name))
+    (minuet--log (format "An error occured when sending request to %s" name)
+                 minuet-show-error-message-on-minibuffer)
     (minuet--log err)))
 
 (defmacro minuet--with-temp-response (&rest body)
